unit cDIBDial;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: cDIBDial.PAS, released August 28, 2000.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2000 Peter Morris.
All Rights Reserved.

Purpose of file:
Dial component

Contributor(s):
None as yet


Last Modified: August 28, 2000
Current Version: 1.00

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org
or at http://www.stuckindoors.com/dib


Known Issues:
To be updated !
-----------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  cDIBImageList, cDIBControl, cDIB, Math;

type
  TDialInc = 1..32767;

  TCustomDIBDial = class(TCustomDIBControl)
  private
    { Private declarations }
    FAutoSize                 : Boolean;
    FBorderSize               : Integer;
    FCapturePointer           : Boolean;
    FPointerAngle             : TAngle;
    FPointerOpacityLow,
    FPointerOpacityHigh       : Byte;
    FIndexMain,
    FIndexPointer             : TDIBImageLink;
    FLastPosition             : Integer;
    FMax,
    FMin                      : Integer;
    FRectMain,
    FRectPointer              : TRect;
    FSmallChange              : TDialInc;
    FPageSize                 : TDialInc;
    FPosition                 : Integer;

    FOnChange                 : TNotifyEvent;
    function  ActualRange : Integer;
    procedure CalcPointerFromPosition;
    procedure CalcPositionFromPointer;
    procedure CalcRects;
    procedure SetMax(const Value: Integer);
    procedure SetMin(const Value: Integer);
    procedure SetPosition(const Value: Integer);
    procedure SetPointerAngle(const Value: TAngle);
    procedure SetPointerOpacityLow(const Value: Byte);
    procedure SetPointerOpacityHigh(const Value: Byte);
    procedure SetAutoSize(const Value: Boolean);
    procedure SetBorderSize(const Value: Integer);
  protected
    { Protected declarations }
    procedure AdjustBounds;
    procedure Change; virtual;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure ImageChanged(Index  : Integer; Operation : TDIBOperation); override;
    procedure Loaded; override;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

    property AutoSize         : Boolean
      read FAutoSize
      write SetAutoSize;
    property BorderSize       : Integer
      read FBorderSize
      write SetBorderSize;
    property IndexMain        : TDIBImageLink
      read FIndexMain
      write FIndexMain;
    property IndexPointer     : TDIBImageLink
      read FIndexPointer
      write FIndexPointer;
    property Max              : Integer
      read FMax
      write SetMax;
    property Min              : Integer
      read FMin
      write SetMin;
    property PointerOpacityLow: Byte
      read FPointerOpacityLow
      write SetPointerOpacityLow;
    property PointerOpacityHigh:Byte
      read FPointerOpacityHigh
      write SetPointerOpacityHigh;
    property PointerAngle  : TAngle
      read FPointerAngle
      write SetPointerAngle;
    property SmallChange      : TDialInc
      read FSmallChange
      write FSmallChange;
    property PageSize         : TDialInc
      read FPageSize
      write FPageSize;
    property Position         : Integer
      read FPosition
      write SetPosition;

    {$I WINControlEvents.inc}
    property OnChange         : TNotifyEvent
      read FOnChange
      write FOnChange;

    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    { Published declarations }
  end;

  TDIBDial = class(TCustomDIBDial)
  published
    property Accelerator;
    property Align;
    property Anchors;
    property AutoSize;
    property BorderSize;
    property DIBFeatures;
    property DIBImageList;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Hint;
    property IndexMain;
    property IndexPointer;
    property Max;
    property Min;
    property Opacity;
    property PageSize;
    property ParentShowHint;
    property PopupMenu;
    property Position;
    property ShowHint;
    property SmallChange;
    property DIBTabOrder;
    property Tag;
    property Visible;

    {$I WinControlEvents.inc}

    property OnChange;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnstartDrag;
  end;

implementation

type
  EDIBDialError = Class(Exception);

{ TCustomDIBDial }

function TCustomDIBDial.ActualRange: Integer;
begin
  Result := Max - Min + 1;
end;

procedure TCustomDIBDial.AdjustBounds;
begin
  if not FIndexMain.Valid then exit;
  
  CalcRects;
  SetBounds(Left,Top,FRectMain.Right - (FRectMain.Left-1),FRectMain.Bottom - (FRectMain.Top-1));
end;

procedure TCustomDIBDial.CalcPointerFromPosition;
begin
  FPointerAngle := Position * 359 div ActualRange;
end;

procedure TCustomDIBDial.CalcPositionFromPointer;
begin
  FPosition := (FPointerAngle * ActualRange div 360) + Min;
end;

procedure TCustomDIBDial.CalcRects;
var
  TheImage                    : TMemoryDIB;
  W,
  H,
  XPos,
  YPos                        : Integer;
begin
  FRectMain := Rect(-1,-1,-1,-1);
  FRectPointer := Rect(-1,-1,-1,-1);

  //Main is the stretchy bit
  if IndexMain.GetImage(TheImage) then begin
    FRectMain.Left := 0;
    FRectMain.Top := 0;
    FRectMain.Right := TheImage.Width;
    FRectMain.Bottom := TheImage.Height;

    XPos := (Width div 2) - (TheImage.Width div 2);
    YPos := (Height div 2) - (TheImage.Height div 2);

    OffsetRect(FRectMain,XPos,YPos);
  end;


  //Pointer is the little twiddly bit
  if IndexPointer.GetImage(TheImage) then begin
    W := (Width div 2) - BorderSize  - (TheImage.Width div 2);
    H := (Height div 2) - BorderSize  - (TheImage.Height div 2);

    FRectPointer.Left :=
      (Width div 2) + (W * SinTable1[FPointerAngle] div 256) - (TheImage.Width div 2);

    FRectPointer.Top :=
      (Height div 2) - (H * CosTable1[FPointerAngle] div 256) - (TheImage.Height div 2);
    FRectPointer.Right := FRectPointer.Left + (TheImage.Width -1);
    FRectPointer.Bottom := FRectPointer.Top + (TheImage.Height -1);
  end;
end;

procedure TCustomDIBDial.Change;
begin
  if FLastPosition <> Position then
    if Assigned(FOnChange) then FOnChange(Self);
  FLastPosition := Position;
end;

constructor TCustomDIBDial.Create(AOwner: TComponent);
begin
  inherited;
  AddIndexProperty(FIndexMain);
  AddIndexProperty(FIndexPointer);
  FRectMain := Rect(-1,-1,-1,-1);
  FRectPointer := Rect(-1,-1,-1,-1);
  FMax := 100;
  FMin := 0;
  FPosition := 0;
  FPointerAngle := 0;
  FCapturePointer := False;
  FPointerOpacityLow := 196;
  FSmallChange := 1;
//  FLargeChange := 5;
  FPageSize := 5;
  FAutoSize := True;
  FBorderSize := 0;

  AddTemplateProperty('AutoSize');
  AddTemplateProperty('BorderSize');
//  AddTemplateProperty('LargeChange');
  AddTemplateProperty('Min');
  AddTemplateProperty('Max');
  AddTemplateProperty('Opacity');
  AddTemplateProperty('PageSize');
  AddTemplateProperty('Position');
  AddTemplateProperty('SmallChange');
end;

destructor TCustomDIBDial.Destroy;
begin
  FIndexMain.Free;
  FIndexPointer.Free;
  inherited;
end;

procedure TCustomDIBDial.DoEnter;
begin
  inherited;
  Invalidate;
end;

procedure TCustomDIBDial.DoExit;
begin
  inherited;
  Invalidate;
end;

procedure TCustomDIBDial.ImageChanged(Index : Integer; Operation: TDIBOperation);
begin
  if AutoSize then
    AdjustBounds
  else
    CalcRects;
end;

procedure TCustomDIBDial.KeyDown(var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  case Key of
    VK_LEFT : Position := Position - SmallChange;
    VK_RIGHT : Position := Position + SmallChange;
    VK_UP : Position := Position + SmallChange;
    VK_DOWN : Position := Position - SmallChange;
    VK_PRIOR : Position := Position + PageSize;
    VK_NEXT : Position := Position - PageSize;
  end;
end;

procedure TCustomDIBDial.Loaded;
begin
  inherited;
  FLastPosition := Position;
  if AutoSize then AdjustBounds;
  CalcPointerFromPosition;
end;

procedure TCustomDIBDial.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if PtInRect(FRectPointer, Point(X,Y)) then begin
    FCapturePointer := True;
    Invalidate;
  end;
end;

procedure TCustomDIBDial.MouseMove(Shift: TShiftState; X,
  Y: Integer);
var
  NewAngle                    : Integer;
begin
  inherited;
  if FCapturePointer then begin
    NewAngle := RelativeAngle((Width div 2), (Height div 2),X,Y);
    PointerAngle := NewAngle;
  end;
end;

procedure TCustomDIBDial.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if FCapturePointer then Invalidate;
  FCapturePointer := False;
  inherited;
end;

procedure TCustomDIBDial.Paint;
var
  TheDIB,
  TempDIB                     : TMemoryDIB;
begin
  if IndexMain.GetImage(TheDIB) then begin
    TempDIB :=
      TMemoryDIB.Create(FRectMain.Right - (FRectMain.Left-1),FRectMain.Bottom - (FRectMain.Top-1));
    try
      TempDIB.StretchCopyPicture(TheDIB);
      TheDIB.AssignHeaderTo(TempDIB);
      TempDIB.Draw(FRectMain.Left,FRectMain.Top,
        FRectMain.Right - (FRectMain.Left -1),FRectMain.Bottom - (FRectMain.Top -1),
        ControlDIB, 0,0);
    finally
      TempDIB.Free;
    end;
  end;
  if IndexPointer.GetImage(TheDIB) then begin
    if FCapturePointer or Focused then
      TheDIB.Opacity := 255
    else
      TheDIB.Opacity := PointerOpacityLow;
    TheDIB.Draw(FRectPointer.Left,FRectPointer.Top,TheDIB.Width,TheDIB.Height,ControlDIB,0,0);
  end;
end;

procedure TCustomDIBDial.SetAutoSize(const Value: Boolean);
begin
  FAutoSize := Value;
  if Value then AdjustBounds;
  CalcRects;
end;

procedure TCustomDIBDial.SetBorderSize(const Value: Integer);
begin
  FBorderSize := Value;
  CalcRects;
  Invalidate;
end;

procedure TCustomDIBDial.SetBounds(ALeft, ATop, AWidth,
  AHeight: Integer);
begin
  inherited;
  CalcRects;
end;

procedure TCustomDIBDial.SetMax(const Value: Integer);
begin
  FMax := Value;
  if Max <= Min then Min := Max-1;
  if Max < Position then Position := Max;
  CalcRects;
  Invalidate;
end;

procedure TCustomDIBDial.SetMin(const Value: Integer);
begin
  FMin := Value;
  if Min >=Max then Max := Min +1;
  if Min > Position then Position := Min;
  CalcRects;
  Invalidate;
end;

procedure TCustomDIBDial.SetPointerOpacityHigh(const Value: Byte);
begin
  FPointerOpacityHigh := Value;
  invalidate;
end;

procedure TCustomDIBDial.SetPointerOpacityLow(const Value: Byte);
begin
  FPointerOpacityLow := Value;
  Invalidate;
end;

procedure TCustomDIBDial.SetPointerAngle(const Value: TAngle);
begin
  FPointerAngle := Value;
  while FPointerAngle > 359 do Dec(FPointerAngle,360);
  while Integer(FPointerAngle) < 0 do Inc(FPointerAngle,360);
  CalcPositionFromPointer;
  Change;
  CalcRects;
  Invalidate;
end;

procedure TCustomDIBDial.SetPosition(const Value: Integer);
begin
  if Value < Min then
    FPosition := Min
  else
  if Value > Max then
    FPosition := Max
  else
    FPosition := Value;

  CalcPointerFromPosition;
  Change;
  CalcRects;
  Invalidate;
end;

end.
